# original work on this was done by @mileszim - https://github.com/mileszim/chroma/tree/cloudflare-workers-ai-embedding
import logging
import os
from typing import Optional, Dict, cast

import httpx

from chromadb import Documents, EmbeddingFunction, Embeddings

logger = logging.getLogger(__name__)


class CloudflareWorkersAIEmbeddings(EmbeddingFunction[Documents]):
    # Follow API Quickstart for Cloudflare Workers AI
    # https://developers.cloudflare.com/workers-ai/
    # Information about the text embedding modules in Google Vertex AI
    # https://developers.cloudflare.com/workers-ai/models/embedding/
    def __init__(
        self,
        model_name: Optional[str] = "@cf/baai/bge-base-en-v1.5",
        *,
        api_token: Optional[str] = os.getenv("CF_API_TOKEN"),
        account_id: Optional[str] = None,
        gateway_endpoint: Optional[
            str
        ] = None,  # use Cloudflare AI Gateway instead of the usual endpoint
        # right now endpoint schema supports up to 100 docs at a time
        # https://developers.cloudflare.com/workers-ai/models/bge-small-en-v1.5/#api-schema (Input JSON Schema)
        max_batch_size: Optional[int] = 100,
        headers: Optional[Dict[str, str]] = None,
    ):
        """
        Initialize the Cloudflare Workers AI Embeddings function.

        :param model_name: The name of the model to use. Defaults to "@cf/baai/bge-base-en-v1.5".
        :param api_token: The API token to use. Defaults to the CF_API_TOKEN environment variable.
        :param account_id: The account ID to use.
        :param gateway_endpoint: The gateway URL to use.
        :param max_batch_size: The maximum batch size to use. Defaults to 100.
        :param headers: The headers to use. Defaults to None.
        """
        if not gateway_endpoint and not account_id:
            raise ValueError(
                "Please provide either an account_id or a gateway_endpoint."
            )
        if gateway_endpoint and account_id:
            raise ValueError(
                "Please provide either an account_id or a gateway_endpoint, not both."
            )
        if gateway_endpoint is not None and not gateway_endpoint.endswith("/"):
            gateway_endpoint += "/"
        self._api_url = (
            f"{gateway_endpoint}{model_name}"
            if gateway_endpoint is not None
            else f"https://api.cloudflare.com/client/v4/accounts/{account_id}/ai/run/{model_name}"
        )
        self._session = httpx.Client()
        self._session.headers.update(headers or {})
        self._session.headers.update({"Authorization": f"Bearer {api_token}"})
        self._max_batch_size = max_batch_size

    def __call__(self, texts: Documents) -> Embeddings:
        # Endpoint accepts up to 100 items at a time. We'll reject anything larger.
        # It would be up to the user to split the input into smaller batches.
        if self._max_batch_size and len(texts) > self._max_batch_size:
            raise ValueError(
                f"Batch too large {len(texts)} > {self._max_batch_size} (maximum batch size)."
            )

        print("URI", self._api_url)

        response = self._session.post(f"{self._api_url}", json={"text": texts})
        response.raise_for_status()
        _json = response.json()
        if "result" in _json and "data" in _json["result"]:
            return cast(Embeddings, _json["result"]["data"])
        else:
            raise ValueError(f"Error calling Cloudflare Workers AI: {response.text}")



import importlib
import logging
import os
from functools import cached_property
from typing import List, Optional, cast

import numpy as np
import numpy.typing as npt
from chromadb.api.types import Documents, EmbeddingFunction, Embeddings

logger = logging.getLogger(__name__)


class OnnxRuntimeEmbeddings(EmbeddingFunction[Documents]):
    def __init__(
        self,
        model_path,
        *,
        preferred_providers: Optional[List[str]] = None,
        max_length: Optional[int] = 256,
        enabled_padding: Optional[bool] = True,
        hf_download: Optional[bool] = False,
        model_cache_dir: Optional[str] = None,
    ) -> None:
        """
        Initialize the OnnxRuntimeEmbeddings.

        :param model_path: This can be a local path to the model or the HuggingFace repository. You need to install huggingface_hub package.
        :param preferred_providers: The preferred providers to use. If not provided, all available providers will be used.
        :param max_length: The maximum length of the input sequence. Default is 256.
        :param enabled_padding: Whether to enable padding. Default is True.
        :param hf_download: Whether to download the model from HuggingFace repository. Default is False.
        """
        if preferred_providers and not all(
            [isinstance(i, str) for i in preferred_providers]
        ):
            raise ValueError("Preferred providers must be a list of strings")
        # check for duplicate providers
        if preferred_providers and len(preferred_providers) != len(
            set(preferred_providers)
        ):
            raise ValueError("Preferred providers must be unique")
        self._preferred_providers = preferred_providers
        self._local_model_path = os.path.expanduser(model_path)
        if hf_download:
            try:
                from huggingface_hub import snapshot_download
            except ImportError:
                raise ValueError(
                    "The `huggingface_hub` python package is not installed. "
                    "Please install it with `pip install huggingface_hub`"
                )
            self._local_model_path = snapshot_download(repo_id=model_path)
        self._actual_model_path = self._find_onnx_model(self._local_model_path)
        self._max_length = max_length
        self._enabled_padding = enabled_padding
        try:
            # Equivalent to import onnxruntime
            self.ort = importlib.import_module("onnxruntime")
        except ImportError:
            raise ValueError(
                "The onnxruntime python package is not installed. Please install it with `pip install onnxruntime`"
            )
        try:
            self.Tokenizer = importlib.import_module("tokenizers").Tokenizer
        except ImportError:
            raise ValueError(
                "The tokenizers python package is not installed. Please install it with `pip install tokenizers`"
            )

    @staticmethod
    def _find_onnx_model(model_path: str) -> str:
        _model_paths = [
            os.path.join(model_path, "model.onnx"),
            os.path.join(model_path, "onnx", "model.onnx"),
        ]
        if not os.path.exists(model_path):
            raise ValueError(f"Model path {model_path} does not exist")
        _actual_model_path = None
        # possible paths for model model_path/model.onnx or model_path/onnx/model.onnx
        for mp in _model_paths:
            if os.path.exists(mp):
                _actual_model_path = mp
                break
        if not _actual_model_path:
            raise ValueError(
                f"Cannot find onnx model un the following paths: {_model_paths}"
            )
        return _actual_model_path

    # Use pytorches default epsilon for division by zero
    # https://pytorch.org/docs/stable/generated/torch.nn.functional.normalize.html
    def _normalize(self, v: npt.NDArray[np.float32]) -> npt.NDArray[np.float32]:
        norm = np.linalg.norm(v, axis=1)
        norm[norm == 0] = 1e-12
        return cast(npt.NDArray[np.float32], v / norm[:, np.newaxis])

    def _forward(
        self, documents: List[str], batch_size: int = 32
    ) -> npt.NDArray[np.float32]:
        all_embeddings = []
        for i in range(0, len(documents), batch_size):
            input_names = [input.name for input in self.model.get_inputs()]
            batch = documents[i : i + batch_size]
            encoded = [self.tokenizer.encode(d) for d in batch]
            input_ids = np.array([e.ids for e in encoded])
            attention_mask = np.array([e.attention_mask for e in encoded])
            onnx_input = {
                "input_ids": np.array(input_ids, dtype=np.int64),
                "attention_mask": np.array(attention_mask, dtype=np.int64),
                "token_type_ids": np.array(
                    [np.zeros(len(e), dtype=np.int64) for e in input_ids],
                    dtype=np.int64,
                ),
            }
            if "token_type_ids" not in input_names:
                del onnx_input["token_type_ids"]
            model_output = self.model.run(None, onnx_input)
            last_hidden_state = model_output[0]
            # Perform mean pooling with attention weighting
            input_mask_expanded = np.broadcast_to(
                np.expand_dims(attention_mask, -1), last_hidden_state.shape
            )
            embeddings = np.sum(last_hidden_state * input_mask_expanded, 1) / np.clip(
                input_mask_expanded.sum(1), a_min=1e-9, a_max=None
            )
            embeddings = self._normalize(embeddings).astype(np.float32)
            all_embeddings.append(embeddings)
        return np.concatenate(all_embeddings)

    @cached_property
    def tokenizer(self) -> "Tokenizer":  # noqa F821
        # TODO is this brittle? Can tokenizer.json be in different place?
        tokenizer = self.Tokenizer.from_file(
            os.path.join(self._local_model_path, "tokenizer.json")
        )
        if self._max_length:
            tokenizer.enable_truncation(max_length=self._max_length)
        if self._enabled_padding:
            tokenizer.enable_padding(length=self._max_length)
        # tokenizer.enable_truncation(max_length=256)
        # tokenizer.enable_padding(pad_id=0, pad_token="[PAD]", length=256)
        return tokenizer

    @cached_property
    def model(self) -> "InferenceSession":  # noqa F821
        if self._preferred_providers is None or len(self._preferred_providers) == 0:
            if len(self.ort.get_available_providers()) > 0:
                logger.debug(
                    f"WARNING: No ONNX providers provided, defaulting to available providers: "
                    f"{self.ort.get_available_providers()}"
                )
            self._preferred_providers = self.ort.get_available_providers()
        elif not set(self._preferred_providers).issubset(
            set(self.ort.get_available_providers())
        ):
            raise ValueError(
                f"Preferred providers must be subset of available providers: {self.ort.get_available_providers()}"
            )

        so = self.ort.SessionOptions()
        sess = self.ort.InferenceSession(
            self._actual_model_path,
            # Since 1.9 onnyx runtime requires providers to be specified when there are multiple available - https://onnxruntime.ai/docs/api/python/api_summary.html
            # This is probably not ideal but will improve DX as no exceptions will be raised in multi-provider envs
            providers=self._preferred_providers,
            sess_options=so,
        )

        return sess

    def __call__(self, input: Documents) -> Embeddings:
        return cast(Embeddings, self._forward(input).tolist())



from enum import Enum
from typing import Union, Sequence, Optional, Any, Dict

from chromadb import Where, WhereDocument


class Query:
    def __init__(self, query: Dict[str, Any]):
        self.q = query

    def to_dict(self) -> Dict[str, Any]:
        return self.q

    def __and__(self, *other: "Query") -> "Query":
        return and_(self, *other)

    def __or__(self, *other: "Query") -> "Query":
        return or_(self, *other)


def where(query: Query) -> Where:
    return query.to_dict()


def where_document(query: Query) -> WhereDocument:
    return query.to_dict()


def eq(attr: str, val: Union[str, int, float, bool]) -> Query:
    return Query({attr: ["$eq", val]})


def lte(attr: str, val: Union[int, float]) -> Query:
    return Query({attr: ["$lte", val]})


def gte(attr: str, val: Union[int, float]) -> Query:
    return Query({attr: ["$gte", val]})


def ne(attr: str, val: Union[str, int, float, bool]) -> Query:
    return Query({attr: ["$ne", val]})


def lt(attr: str, val: Union[int, float]) -> Query:
    return Query({attr: ["$lt", val]})


def gt(attr: str, val: Union[int, float]) -> Query:
    return Query({attr: ["$gt", val]})


def in_(attr: str, val: Sequence[Union[str, int, float, bool]]) -> Query:
    return Query({attr: ["$in", val]})


def nin(attr: str, val: Sequence[Union[str, int, float, bool]]) -> Query:
    return Query({attr: ["$nin", val]})


def and_(*args: Query) -> Query:
    return Query({"$and": [a.to_dict() for a in args]})


def or_(*args: Query) -> Query:
    return Query({"$or": [a.to_dict() for a in args]})


class LogicalOperator(str, Enum):
    AND = "$and"
    OR = "$or"


def contains(*val: str, op: Optional[LogicalOperator] = LogicalOperator.AND) -> Query:
    if len(val) == 1:
        return Query({"$contains": val[0]})
    if op is LogicalOperator.AND:
        return Query({"$and": [{"$contains": v} for v in val]})
    return Query({"$or": [{"$contains": v} for v in val]})


def not_contains(
    *val: str, op: Optional[LogicalOperator] = LogicalOperator.AND
) -> Query:
    if len(val) == 1:
        return Query({"$not_contains": val[0]})
    if op is LogicalOperator.AND:
        return Query({"$and": [{"$not_contains": v} for v in val]})
    return Query({"$or": [{"$not_contains": v} for v in val]})



import os.path
from enum import Enum
from typing import Optional

from chromadb import EmbeddingFunction, Documents


class PoolingType(int, Enum):
    NONE = 0
    MEAN = 1
    CLS = 2
    LAST = 3


class LlamaCppEmbeddingFunction(EmbeddingFunction[Documents]):
    def __init__(
        self,
        model_path: str,
        *,
        hf_file_name: Optional[str] = None,
        pooling_type: Optional[PoolingType] = PoolingType.MEAN,
    ) -> None:
        """
        Initialize the LlamaCppEmbeddingFunction.

        :param model_path: This can be a local path to the model or the HuggingFace repository. You need to install huggingface_hub package.
        :param hf_file_name: The name of the file in the HuggingFace repository.
            This is only required if the model_path is a HuggingFace repository.
        :param pooling_type: The pooling type to use. Default is `PoolingType.MEAN`.
        """
        try:
            from llama_embedder import LlamaEmbedder, PoolingType as PT
        except ImportError:
            raise ValueError(
                "The `llama-embedder` python package is not installed. "
                "Please install it with `pip install llama-embedder`"
            )

        if not os.path.exists(model_path) and hf_file_name is None:
            raise ValueError(f"Model path {model_path} does not exist")
        elif os.path.exists(model_path):
            self._model_file = model_path
        elif model_path and hf_file_name:
            try:
                from huggingface_hub import hf_hub_download
            except ImportError:
                raise ValueError(
                    "The `huggingface_hub` python package is not installed. "
                    "Please install it with `pip install huggingface_hub`"
                )
            self._model_file = hf_hub_download(
                repo_id=model_path, filename=hf_file_name
            )
        if pooling_type is None:
            pt = PT.NONE
        elif pooling_type == PoolingType.MEAN:
            pt = PT.MEAN
        elif pooling_type == PoolingType.CLS:
            pt = PT.CLS
        elif pooling_type == PoolingType.LAST:
            pt = PT.LAST
        else:
            raise ValueError(f"Invalid pooling type: {pooling_type}")

        self._embedder = LlamaEmbedder(model_path=self._model_file, pooling_type=pt)

    def __call__(self, input: Documents) -> Optional[Documents]:
        return self._embedder.embed(input)



from typing import Optional, cast

from chromadb.api.types import Documents, EmbeddingFunction, Embeddings


class GoogleVertexAiEmbeddings(EmbeddingFunction[Documents]):
    def __init__(
        self,
        model_name: str = "text-embedding-004",
        *,
        project_id: Optional[str] = None,
        location: Optional[str] = None,
        dimensions: Optional[int] = 256,
        task_type: Optional[str] = "RETRIEVAL_DOCUMENT",
        credentials: Optional[any] = None,
        api_key: Optional[str] = None,
        api_endpoint: Optional[str] = None,
        api_transport: Optional[str] = None,
    ) -> None:
        """
        Initialize the GoogleVertexAi.

        :param model_name: The name of the model to use. Defaults to "text-embedding-004".
        :param project_id: The project ID to use. Defaults to None.
        :param location: The location to use. Defaults to None.
        :param dimensions: The number of dimensions to use. Defaults to None.
        :param task_type: The task type to use. Defaults to "RETRIEVAL_DOCUMENT". https://cloud.google.com/vertex-ai/generative-ai/docs/embeddings/task-types
        :param credentials: The credentials to use. Defaults to None.
        :param api_key: The API key to use. Defaults to None.
        :param api_endpoint: The API endpoint to use. Defaults to None.
        :param api_transport: The API transport to use. Defaults to None.
        """
        try:
            import vertexai
            from vertexai.language_models import TextEmbeddingModel

            vertexai.init(
                project=project_id,
                location=location,
                credentials=credentials,
                api_key=api_key,
                api_endpoint=api_endpoint,
                api_transport=api_transport,
            )
            self._model = TextEmbeddingModel.from_pretrained(model_name)
        except ImportError:
            raise ValueError(
                "The vertexai python package is not installed. Please install it with `pip install vertexai`"
            )
        self._dimensions = dimensions
        self._task_type = task_type

    def __call__(self, input: Documents) -> Embeddings:
        from vertexai.language_models import TextEmbeddingInput

        inputs = [TextEmbeddingInput(text, self._task_type) for text in input]
        kwargs = (
            dict(output_dimensionality=self._dimensions) if self._dimensions else {}
        )
        embeddings = [
            embedding.values
            for embedding in self._model.get_embeddings(inputs, **kwargs)
        ]
        return cast(Embeddings, embeddings)



import hashlib
import os
import uuid
from functools import partial
from typing import Generator, Callable, Optional

from chromadb import IDs, Documents


def uuid_id_generator() -> Generator[str, None, None]:
    while True:
        yield f"{uuid.uuid4()}"


class IDGenerator(IDs):
    def __init__(
        self,
        ids_len: int,
        generator: Optional[
            Callable[..., Generator[str, None, None]]
        ] = uuid_id_generator,
    ):
        """
        Parameters:
            ids_len (int):  The number of IDs to generate. This must be equal to the number of documents.
            generator (callable):  The function to generate the IDs. The default is uuid_id_generator.

        Example:
            Here's how to use the IDGenerator class:

                >>> import chromadb
                >>> from chromadbx import IDGenerator
                >>> client = chromadb.Client()
                >>> col = client.get_or_create_collection("test")
                >>> my_docs = [f"Document {i}" for i in range(10)]
                >>> col.add(ids=IDGenerator(len(my_docs)), documents=my_docs)

            The above code will generate UUIDs for each document.
        """
        self.ids_len = ids_len
        self._generator = generator()
        self._items = list(next(self._generator) for _ in range(ids_len))

    def __len__(self):
        return self.ids_len

    def __getitem__(self, index):
        return self._items[index]

    def __iter__(self):
        return iter(self._items)

    def __json__(self):
        return self._items


class UUIDGenerator(IDGenerator):
    def __init__(self, ids_len: int):
        """
        Parameters:
            ids_len (int):  The number of IDs to generate. This must be equal to the number of documents.

        Example:
            Here's how to use the UUIDIDGenerator class:

                >>> import chromadb
                >>> from chromadbx import UUIDGenerator
                >>> client = chromadb.Client()
                >>> col = client.get_or_create_collection("test")
                >>> my_docs = [f"Document {i}" for i in range(10)]
                >>> col.add(ids=UUIDGenerator(len(my_docs)), documents=my_docs)

            The above code will generate UUIDs for each document.
        """
        super().__init__(ids_len=ids_len, generator=uuid_id_generator)


def generate_random_sha256_hash() -> Generator[str, None, None]:
    while True:
        # Generate a random number
        random_data = os.urandom(16)
        # Create a SHA256 hash object
        sha256_hash = hashlib.sha256()
        # Update the hash object with the random data
        sha256_hash.update(random_data)
        # Yield the hexadecimal representation of the hash
        yield sha256_hash.hexdigest()


class RandomSHA256Generator(IDGenerator):
    def __init__(self, ids_len: int):
        """
        Parameters:
            ids_len (int):  The number of IDs to generate. This must be equal to the number of documents.

        Example:
            Here's how to use the SHA256IDGenerator class:

                >>> import chromadb
                >>> from chromadbx import RandomSHA256Generator
                >>> client = chromadb.Client()
                >>> col = client.get_or_create_collection("test")
                >>> my_docs = [f"Document {i}" for i in range(10)]
                >>> col.add(ids=RandomSHA256Generator(len(my_docs)), documents=my_docs)

            The above code will generate SHA256 hashes for each document.
        """
        super().__init__(ids_len=ids_len, generator=generate_random_sha256_hash)


def generate_documents_sha256_hash(documents: Documents) -> Generator[str, None, None]:
    for doc in documents:
        # Create a SHA256 hash object
        sha256_hash = hashlib.sha256()
        # Update the hash object with the random data
        sha256_hash.update(doc.encode("utf-8"))
        # Yield the hexadecimal representation of the hash
        yield sha256_hash.hexdigest()


class DocumentSHA256Generator(IDGenerator):
    def __init__(self, documents: Documents):
        """
        Parameters:
            documents (Documents):  The documents to generate SHA256 hashes for.

        Example:
            Here's how to use the DocumentSHA256IDGenerator class:

                >>> import chromadb
                >>> from chromadbx import DocumentSHA256Generator
                >>> docs = ["Document 1", "Document 2", "Document 3"]
                >>> client = chromadb.Client()
                >>> col = client.get_or_create_collection("test")
                >>> col.add(ids=DocumentSHA256Generator(docs), documents=docs)

            The above code will generate SHA256 hashes for each document.
        """
        super().__init__(
            ids_len=len(documents),
            generator=partial(generate_documents_sha256_hash, documents=documents),
        )


def ulid_generator() -> Generator[str, None, None]:
    try:
        import ulid
    except ImportError:
        raise ValueError(
            "The ulid python package is not installed. Please install it with `pip install ulid-py`"
        )
    while True:
        yield str(ulid.new())


class ULIDGenerator(IDGenerator):
    def __init__(self, ids_len: int):
        """
        Parameters:
            ids_len (int):  The number of IDs to generate. This must be equal to the number of documents.

        Example:
            Here's how to use the ULIDGenerator class:

                >>> import chromadb
                >>> from chromadbx import ULIDGenerator
                >>> client = chromadb.Client()
                >>> col = client.get_or_create_collection("test")
                >>> my_docs = [f"Document {i}" for i in range(10)]
                >>> col.add(ids=ULIDGenerator(len(my_docs)), documents=my_docs)

            The above code will generate ULIDs for each document.
        """
        super().__init__(ids_len=ids_len, generator=ulid_generator)


def nano_id_generator(
    alphabet: Optional[str] = None, size: Optional[int] = None
) -> Generator[str, None, None]:
    try:
        from nanoid import generate
        import nanoid.resources
    except ImportError:
        raise ValueError(
            "The nanoid python package is not installed. Please install it with `pip install nanoid`"
        )
    while True:
        yield str(
            generate(
                alphabet=alphabet or nanoid.resources.alphabet,
                size=size or nanoid.resources.size,
            )
        )


class NanoIDGenerator(IDGenerator):
    def __init__(
        self, ids_len: int, alphabet: Optional[str] = None, size: Optional[int] = None
    ):
        """
        Parameters:
            ids_len (int):  The number of IDs to generate. This must be equal to the number of documents.
            alphabet (str):  The alphabet to use for generating the IDs. The default is `_-0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ`.
            size (int):  The size of the IDs to generate. The default is 21.

        Example:
            Here's how to use the NanoIDGenerator class:

                >>> import chromadb
                >>> from chromadbx import NanoIDGenerator
                >>> client = chromadb.Client()
                >>> col = client.get_or_create_collection("test")
                >>> my_docs = [f"Document {i}" for i in range(10)]
                >>> col.add(ids=NanoIDGenerator(len(my_docs)), documents=my_docs)

            The above code will generate NanoIDs for each document.
        """
        super().__init__(
            ids_len=ids_len,
            generator=partial(nano_id_generator, alphabet=alphabet, size=size),
        )



import os
from typing import Optional, cast

from chromadb.api.types import Documents, EmbeddingFunction, Embeddings


class MistralAiEmbeddings(EmbeddingFunction[Documents]):
    def __init__(
        self,
        model_name: str = "mistral-embed",
        *,
        api_key: Optional[str] = os.getenv("MISTRAL_API_KEY"),
        retries: Optional[int] = None,
    ) -> None:
        """
        Initialize the Mistral AI EF.

        :param model_name: The name of the model to use. Defaults to "mistral-embed".
        :param api_key: The API key
        :param retries: The number of retries to use. Defaults to None.
        """
        try:
            from mistralai import Mistral

            self._client = Mistral(api_key=api_key)
            self._model = model_name
            self._retries = retries
        except ImportError:
            raise ValueError(
                "The mistralai python package is not installed. Please install it with `pip install mistralai`"
            )

    def __call__(self, input: Documents) -> Embeddings:
        embeddings_batch_response = self._client.embeddings.create(
            model=self._model,
            inputs=input,
        )
        embeddings = [d.embedding for d in embeddings_batch_response.data]
        return cast(Embeddings, embeddings)



try:
    import chromadb  # noqa: F401
except ImportError:
    raise ValueError(
        "The chromadb is not installed. This package (chromadbx) requires that Chroma is installed to work. "
        "Please install it with `pip install chromadb`"
    )

from chromadbx.core.ids import (
    IDGenerator,
    NanoIDGenerator,
    ULIDGenerator,
    DocumentSHA256Generator,
    RandomSHA256Generator,
    UUIDGenerator,
)
from chromadbx.core.queries import (
    where,
    where_document,
    eq,
    lte,
    gte,
    ne,
    lt,
    gt,
    in_,
    nin,
    and_,
    or_,
    LogicalOperator,
    not_contains,
    contains,
)

__all__ = [
    "IDGenerator",
    "UUIDGenerator",
    "NanoIDGenerator",
    "ULIDGenerator",
    "DocumentSHA256Generator",
    "RandomSHA256Generator",
    "where",
    "where_document",
    "eq",
    "lte",
    "gte",
    "ne",
    "lt",
    "gt",
    "in_",
    "nin",
    "and_",
    "or_",
    "contains",
    "not_contains",
    "LogicalOperator",
]
